# !/usr/bin/env python
# encoding: utf-8
"""
$Id$

Purpose: Provide interface to bash ancillary retrieval scripts.

Input:

Output:


Details:

   Main entry point-   service_remote_ancillary(work_dir,granules_to_process, ancillary_type )
   Given the directory containg the spacecraft data, the science granules and type of ancillary data.
   Return a list of the required ancillary files.

Preconditions:



Copyright (c) 2011 University of Wisconsin Regents.
Licensed under GNU GPLv3.
"""
import os
import logging
import glob
from datetime import datetime, timedelta
from adl_common import CSPP_RT_HOME
from ancillary_control import retreive_rsbautocal, retreive_polar, retreive_tle, retreive_cmo, retreive_fs_cmo

from npp_orb_num import NppOrbNum
from adl_query_dms import update_orbit_number
import pprint
from stat import *

pp = pprint.PrettyPrinter(indent=4)
# every module should have a LOG object
LOG = logging.getLogger(__name__)

# add ancillary scripts by
# define constant identifier
# define command string
# add identifier and command to dictionary

kPOLAR = "POLAR"
kTLE = "TLE"
kGRIB = "GRIB"
kNISE = "NISE"
kRSB = 'RSB'
kCMO = 'CMO'

# format strings for time value conversions to shell script parameters
ancillary_strftime = {kPOLAR: "%Y%j %H%M",
                      kTLE: "%Y%j",
                      kGRIB: "%Y%m%d",
                      kNISE: "%Y%j",
                      kRSB: "%Y%j %H:%M:%S",
                      kCMO:  "%Y%j"
                      }

#CrIS-FS-Correct-Matrix-AUX_j01_20180126190335Z_20180121235510Z_ee00000000000000Z_-_cspp_dev_all-_dev.h5

class NppFileNameParser(object):
    def time_range(self, file_name):
        day, begin_time, end_time = self.extract_time_range_tokens(file_name)
        return [self.time_from_tokens(day, t) for t in (begin_time, end_time)]

    def extract_time_range_tokens(self, file_name):
        return file_name.split('_')[2:5]

    def time_from_tokens(self, day, time):
        return datetime.strptime(day[1:] + time[1:], '%Y%m%d%H%M%S%f')


def get_orbit_number(work_dir, h5_file):
    orb_num = 0

    npp_orb = NppOrbNum()

    nppname = NppFileNameParser()
    # LOG.info( nppname.time_range(h5_file) )
    dts = nppname.time_range(os.path.basename(h5_file))
    tle_files = glob.glob(os.path.join(work_dir, 'TLE-AUX*'))

    for tle in tle_files:
        this_orb = npp_orb.usedt(dts[0], tle)
        if this_orb > orb_num:
            orb_num = this_orb
            LOG.debug("Orbit %s" % orb_num)

    return orb_num


def update_orbit_in_db(work_dir, granules_to_process, spcrft):
    """
    """
    # TATMS_npp_      d20170206_t1950373_e1951089_b00001_c20170223221348501244_aau_dev.h5
    # RATMS-RNSCA_npp_d20170206_t1943395_e1955555_b00001_c20170209153509259000_all-_dev.h5
    npp_orb = NppOrbNum(spcrft,work_dir)
    orbit_number = 1
    for gran in granules_to_process:
        LOG.debug(pp.pformat(gran))
        start = gran['RANGE_START_IET']
        tle_files = sorted(glob.glob(os.path.join(work_dir, 'TLE-AUX*')))
        for tle in tle_files:
            #perm= S_IRUSR | S_IWUSR | S_IXUSR
            #LOG.info('mode %s '%tle)
            #os.chmod(tle, perm)

            orbit_number = npp_orb.usedt(start, tle)
            LOG.debug("Orbit %s tle %s" % (orbit_number,tle))

        update_orbit_number(orbit_number, gran['GRANULE_ID'])


def service_remote_ancillary(work_dir, granules_to_process, ancillary_type, spcrft, allow_cache_update = True):
    """
    Check the DB ancillary site and update ancillary files if needed.  Return a list of
    file names to use with these granules.

    work_dir - directory contain input data.
    granules_to_process - List of granules names
    ancillary type - String defining type of ancillary data
    """
    time_fmt = ancillary_strftime[ancillary_type]

    ancillary_file_set = set()
    date_set = set()

    for gran_name in granules_to_process:
        startTimeDateObj = gran_name['StartTime']
        startTimeDateObj = startTimeDateObj.replace(
            hour=0, minute=0, second=0, microsecond=1)
        #startTimeDateObj = startTimeDateObj - timedelta(days=1)

        date = startTimeDateObj.strftime(time_fmt)
        date_set.add(date)

    LOG.debug("dates of interest: %r" % date_set)

    LYJ = "0"

    for date in sorted(date_set):

        YJ = date.split(' ')[0]

        if YJ != LYJ:
            LOG.debug('recheck %s %s' % (YJ, LYJ))
            LYJ = YJ

            desired_time = datetime.strptime(date, time_fmt)
            if ancillary_type == kRSB:
                needed_ancillary_files = retreive_rsbautocal(
                    work_dir, desired_time, spcrft, allow_remote=allow_cache_update)

            if ancillary_type == kPOLAR:
                needed_ancillary_files = retreive_polar(
                    work_dir, desired_time,  allow_remote=allow_cache_update)

            if ancillary_type == kTLE:
                needed_ancillary_files = retreive_tle(
                    work_dir, desired_time, spcrft, allow_remote=allow_cache_update)
                
            if ancillary_type == kCMO :
                needed_ancillary_filesa = retreive_cmo(
                    work_dir, desired_time, spcrft, allow_remote=allow_cache_update)
                 
                needed_ancillary_filesb = retreive_fs_cmo(work_dir, desired_time, spcrft, allow_remote=allow_cache_update)
                needed_ancillary_files = needed_ancillary_filesa + needed_ancillary_filesb
    
            LOG.info('files back %s %s' %
                     (str(ancillary_type), str(needed_ancillary_files)))
            
            if len(needed_ancillary_files) == 0:
                LOG.error('Critical file missing: %s'%ancillary_type)
                #if ancillary_type == kTLE:
                #   raise RuntimeError('Critical file missing: %s'%ancillary_type)

            for pathname in needed_ancillary_files:
                ancillary_file_set.add(pathname)
        else:
            pass


    LOG.debug("ancillary file set: %r" % ancillary_file_set)

    return list(sorted(ancillary_file_set))
