#!/usr/bin/env python
# encoding: utf-8

"""
$Id$

Purpose: Common routines for ADL XDR handling and ancillary data caching.

Requires: adl_asc

Created Oct 2011 by R.K.Garcia <rayg@ssec.wisc.edu>
Copyright (c) 2011 University of Wisconsin Regents.
Licensed under GNU GPLv3.
"""

import os
import sys
import logging
import glob
import traceback
import time
import pprint
import fileinput
import re
from subprocess import Popen, CalledProcessError, call, PIPE, run
import datetime 
import datetime as dt

import h5py
from adl_asc import corresponding_asc_path
#import shutil
import tempfile
#from matplotlib.cm import LUTSIZE
LOG = logging.getLogger(__name__)
pp = pprint.PrettyPrinter(indent=4)

# Table of common ADL SDR log error messages and the associated hint to correcting the problem.
# Algorithm-specific error messages should not go here.
COMMON_LOG_CHECK_TABLE = [
    ("ERROR - CMN GEO set up ephemeris and attitude failure",
     "Check Polar and TLE data"),
    ("ERROR - CMN GEO satellite position and attitude failure",
     "Problem with S/C Diary"),
    ("PRO_FAIL Required input not available",
     "Missing or out of date ancillary input"),
    ("PRO_FAIL runAlgorithm()", "Algorithm failed"),
    ("Completed unsuccessfully", "Algorithm failed"),
    ("The DMS directory is not valid:",
     "Is anc/static data installed?, Is anc/cache data installed? Check DMS path setting in configuration."),
    ("arbitrary time is invalid",
     "Problem with input RDR, check NPP_GRANULE_ID_BASETIME"),
    ("Error retrieving data for USNO-POLARWANDER-UT1",
     "POLAR WANDER file needs update, check NPP_GRANULE_ID_BASETIME"),
    ("Verify that the correct TLE data file has been retrieved",
        "TLE file needs update, check NPP_GRANULE_ID_BASETIME"),
    ( "TLE epoch time is greater than 30 days","TLE file needs update"),
]

PROFILING_ENABLED = os.environ.get('CSPP_PROFILE', None) is not None
STRACE_ENABLED = os.environ.get('CSPP_STRACE', None) is not None

PACKAGE_DATABASE = 'cspp_sdr_package.db'
def get_shipped_db(cspp_sdr_home):
    package_dms_path = os.path.join(cspp_sdr_home, 'anc', 'cache', 'package')
    initial_db = os.path.join(package_dms_path, PACKAGE_DATABASE)

    return initial_db

# ref:
# http://stackoverflow.com/questions/1383254/logging-streamhandler-and-standard-streams
class SingleLevelFilter(logging.Filter):

    def __init__(self, passlevels, reject):
        self.passlevels = set(passlevels)
        self.reject = reject

    def filter(self, record):
        if self.reject:
            return (record.levelno not in self.passlevels)
        else:
            return (record.levelno in self.passlevels)


def _ldd_verify(exe):
    "check that a program is ready to run"
    # rc = call(['ldd', exe], stdout=os.tmpfile(), stderr=os.tmpfile())
    rc = call(['ldd', exe], stdout=tempfile.TemporaryFile(),
              stderr=tempfile.TemporaryFile())
    return (rc == 0)


def split_search_path(s):
    "break a colon-separated list of directories into a list of directories, else empty-list"
    if not s:
        return []

    back_list = []
    for path in s.split(':'):
        back_list.append(os.path.abspath(path))

    return back_list


def _replaceAll(infile, searchExp, replaceExp):
    for line in fileinput.input(infile, inplace=1):
        if searchExp in line:
            line = line.replace(searchExp, replaceExp)
        sys.stdout.write(line)
    fileinput.close()

class AscLineParser(object):

    def time_range(self, ascLine):
        day, time = self.extract_time_range_tokens(ascLine)
        return self.time_from_tokens(day, time)

    def extract_time_range_tokens(self, ascLine):
        return ascLine.split('"')[3:4][0].split(' ')

    def time_from_tokens(self, day, time):
        dt = datetime.datetime.strptime(day + time, '%Y-%m-%d%H:%M:%S.%f')

        return dt


def _testParser():
    dt = AscLineParser().time_range(
        '("RangeDateTime" DATETIMERANGE EQ "2014-01-13 11:22:39.900000" "2014-01-13 11:22:59.900000")')
    print (dt)


def check_and_convert_path(key, a_path, check_write=False):
    """
    Make sure the path or paths specified exist
    Return the path or list of absolute paths that exist
    """

    PY3 = sys.version_info[0] == 3

    if PY3:
        string_types = str,
    else:
        string_types = basestring,

    abs_locations = []
    if ":" in a_path:
        paths = a_path.split(":")
    # old elif isinstance(a_path, types.StringTypes) == True:
    elif isinstance(a_path, string_types) == True:
        paths = [a_path]
    else:
        paths = a_path

    for path in paths:
        if not os.path.exists(path):
            LOG.error("Environment variable %s refers to a path that does not exists.  %s=%s" % (
                key, key, path))
            LOG.error(
                "Make sure package HOME variable is set and the package home environment script is sourced.")
            sys.exit(2)
        else:
            LOG.debug("Found: %s at %s %s" %
                      (key, path, os.path.abspath(path)))
            abs_locations.append(os.path.abspath(path))

        if check_write is True:
            if not os.access(path, os.W_OK):
                LOG.error("Path exists but is not writable %s=%s" %
                          (key, path))
                sys.exit(2)

    # return a string if only one and an array if more
    if len(abs_locations) == 1:
        return abs_locations[0]
    else:
        # return abs_locations
        # return a :-joined string for use in an env variable
        return ':'.join(abs_locations)


def check_existing_env_var(varname, default_value=None,silent=False):
    """
    Check for vaiable if it exists use vale otherwise use default
    """
    value = None

    if varname in os.environ:
        value = os.environ.get(varname)
        if not silent :
            LOG.info('environment variable %s is set %s' % (varname, value))
    else:
        if default_value is not None:
            value = default_value
            LOG.debug('set environment variable %s  %s' % (varname, value))

        else:
            LOG.error(
                'ERROR: %s is not set, please update environment and re-try' % varname)
            LOG.error("Environment variable missing. %s" % varname)
            sys.exit(9)

    return value


def check_and_convert_env_var(varname, check_write=False, default_value=None,silent=False):
    value = check_existing_env_var(varname, default_value=default_value,silent=silent)
    path = check_and_convert_path(varname, value, check_write=check_write)
    return path


def what_package_am_i():
    path = os.path.dirname(os.path.abspath(__file__))
    cspp_x = path.split("/common")
    cspp_x_home = cspp_x[0]

    return cspp_x_home


ADL_VARS = {}
EXTERNAL_BINARY = {}
DDS_PRODUCT_FILE = None
ADL_HOME = None


def set_db_name(db_name):
    global ADL_VARS
    ADL_VARS['DB_NAME'] = db_name


def initialize_work_ADL_variables(work_dir, full_db_name):
    global ADL_VARS

    LOG.info('w %s db %s' % (work_dir, full_db_name))
    default = os.path.dirname(full_db_name)

    ADL_VARS['DPE_DMS_INST_PATH'] = check_existing_env_var(
        'DPE_DMS_INST_PATH', default_value=default)

    ADL_VARS['INFTK_DM_ROOT'] = ADL_VARS['DPE_DMS_INST_PATH']

    ADL_VARS['DPE_ROOT_PATH'] = work_dir

    ADL_VARS['DB_NAME'] = check_existing_env_var(
        'DB_NAME', default_value=full_db_name)

    default = os.path.join(work_dir, 'log')
    ADL_VARS['ADL_LOG_LOC'] = check_existing_env_var(
        'ADL_LOG_LOC', default_value=default)

    ADL_VARS['ING_MSD_LZ_PATH'] = check_existing_env_var(
        'ING_MSD_LZ_PATH', default_value=os.path.join(work_dir, 'msd'))

    #export ADL_DB_CONFIG="PRAGMA journal_mode=WAL; PRAGMA synchronous=0;"
    ADL_VARS['ADL_DB_CONFIG'] = "PRAGMA journal_mode=OFF;PRAGMA synchronous=0;"

    ADL_VARS['N_IDPS_MODE'] = "DirectBroadcast"

def initialize_ADL_variables(cspp_xdr_home):
    global ADL_HOME
    global DDS_PRODUCT_FILE
    global ADL_VARS
    
    default = os.path.join(cspp_xdr_home, "common", "ADL")

    ADL_HOME = check_and_convert_env_var(
        'ADL_HOME', check_write=False, default_value=default)
    ADL_VARS['ADL_HOME'] = ADL_HOME

    default = os.path.join(cspp_xdr_home, "common", "ADL", "cfg")
    cfg_dir = check_and_convert_env_var('DCONFIG', default_value=default)
    ADL_VARS['DCONFIG'] = cfg_dir

    # table of NPP short names to data product ids
    DDS_PRODUCT_FILE = os.path.join(cfg_dir, "DDS_PRODUCT_IDS.xml")
    # DPE domain and site id, used in HDF5 filenames

    ADL_VARS['DPE_VER'] = check_existing_env_var(
        'DPE_VER', default_value='CSPP')

    # set the launch base-time for NPP
    ADL_VARS['NPP_GRANULE_ID_BASETIME'] = check_existing_env_var('NPP_GRANULE_ID_BASETIME',
                                                                 default_value="1698019234000000")

    ADL_VARS['J01_GRANULE_ID_BASETIME'] = check_existing_env_var('J01_GRANULE_ID_BASETIME',
                                                                 default_value="1698019234000000")

    ADL_VARS['GW1_GRANULE_ID_BASETIME'] = check_existing_env_var('GW1_GRANULE_ID_BASETIME',
                                                                 default_value="1698019234000000")

    default = os.path.join(cspp_xdr_home, "common", "ADL", "cfg")
    # IET time ancillary used by ADL
    ADL_VARS['DSTATICDATA'] = check_existing_env_var(
        'DSTATICDATA', default_value=default)

    default = os.path.join(
        cspp_xdr_home, "common", "ADL", "cfg", "GuideList.txt")
    ADL_VARS['INFUTIL_CFGDOMAIN'] = check_existing_env_var(
        'INFUTIL_CFGDOMAIN', default_value=default)
    ADL_VARS['DMSUTIL_CFGDOMAIN'] = check_existing_env_var(
        'DMSUTIL_CFGDOMAIN', default_value=default)

    ADL_VARS['COTS_ACROREAD'] = "NOT_USED_IN_ADL"
    ADL_VARS['COTS_PDF_VIEWER'] = "NOT_USED_IN_ADL"
    ADL_VARS['COTS_OPENVIEW_HM'] = "NOT_USED_IN_ADL"
    ADL_VARS['COTS_OKULAR'] = "NOT_USED_IN_ADL"
    ADL_VARS['DOMAIN_CONTROLLER'] = "NOT_USED_IN_ADL"
    ADL_VARS['DPE_INF_STATUS_LOGS_LZ_PATH'] = "NOT_USED_IN_ADL"
    ADL_VARS['DPE_INF_STDOUT_LOGS_LZ_PATH'] = "NOT_USED_IN_ADL"
    ADL_VARS['DPE_SEC_PATH'] = "NOT_USED_IN_ADL"
    ADL_VARS['DPE_USER'] = "NOT_USED_IN_ADL"
    ADL_VARS['LDAP_OVER_SSL'] = "NOT_USED_IN_ADL"
    ADL_VARS['LDAP_USERNAME'] = "NOT_USED_IN_ADL"
    ADL_VARS['PDR_TNS_NAME'] = "NOT_USED_IN_ADL"
    ADL_VARS['SCHED_RMI_PORT'] = "NOT_USED_IN_ADL"
    ADL_VARS['SEARCH_BASE'] = "NOT_USED_IN_ADL"
    ADL_VARS['TWO_TASK'] = "NOT_USED_IN_ADL"
    ADL_VARS['WFM_MONITOR_PORT'] = "NOT_USED_IN_ADL"
    ADL_VARS['MOM_HOST'] = "UNKNOWN"
    ADL_VARS['DDS_FORMAT_RAMDISK'] = "FALSE"
    ADL_VARS['HOST'] = '1234'
    ADL_VARS['DPE_SOFTWARE_VERSION'] = ADL_VARS['DPE_VER']
    
    ADL_VARS['DPE_ORIGIN'] = check_existing_env_var(
        'DPE_SITE_ID', default_value='cspp')
    
    ADL_VARS['DPE_DOMAIN'] = check_existing_env_var(
        'DPE_DOMAIN', default_value='ops')
   
    ADL_VARS['DPE_SUBDOMAIN'] = ''

    for key in iter(ADL_VARS.keys()):
        os.environ[key] = ADL_VARS[key]
        LOG.info("Var " + os.environ[key])

    EXTERNAL_BINARY['ADL_CRIS_SDR'] = os.path.join(
        ADL_HOME, 'bin', 'ProSdrCrisController.exe')
    EXTERNAL_BINARY['ADL_ATMS_SDR'] = os.path.join(
        ADL_HOME, 'bin', 'ProSdrAtmsController.exe')
    EXTERNAL_BINARY['ADL_UNPACKER'] = os.path.join(
        ADL_HOME, 'tools', 'bin', 'ADL_Unpacker.exe')
    EXTERNAL_BINARY['INGMSD'] = os.path.join(
        ADL_HOME, 'tools', 'bin', 'IngMsd.exe')
    #EXTERNAL_BINARY['DBHMICLI'] = os.path.join(
    #    ADL_HOME, 'tools', 'bin', 'DmHmiCLI.exe')
    EXTERNAL_BINARY['URIDCREATOR'] = os.path.join(
        ADL_HOME, 'tools', 'bin', 'UridCreator.exe')

    #EXTERNAL_BINARY['GRANID'] = os.path.join(
    #    ADL_HOME, 'tools', 'bin', 'granID.exe')

CSPP_RT_ANC_CACHE_DIR = None
CSPP_RT_ANC_HOME = None
JPSS_REMOTE_ANC_DIR = None

CSPP_PATHS = {}


def check_anc_cache(check_write=True):
    global CSPP_RT_ANC_CACHE_DIR
    default = os.path.join(CSPP_RT_HOME, "anc/cache")
    CSPP_RT_ANC_CACHE_DIR = check_and_convert_env_var('CSPP_RT_ANC_CACHE_DIR', check_write=check_write,
                                                      default_value=default)
    CSPP_DYNAMIC_ANCIL_DIR = check_and_convert_env_var('CSPP_DYNAMIC_ANCIL_DIR', 
                                                        check_write=check_write,
                                                        default_value=CSPP_RT_ANC_CACHE_DIR)



def initialize_cspp_variables(cspp_home):
    global CSPP_RT_ANC_CACHE_DIR
    global CSPP_RT_ANC_HOME
    global NAGG
    global REPACK
    global JPSS_REMOTE_ANC_DIR

    global CSPP_PATHS
    global EXTERNAL_BINARY

    default = os.path.join(cspp_home, "anc/cache")
    CSPP_RT_ANC_CACHE_DIR = check_and_convert_env_var(
        'CSPP_RT_ANC_CACHE_DIR', check_write=False, default_value=default)

    os.environ['CSPP_DYNAMIC_ANCIL_DIR'] = check_and_convert_env_var('CSPP_DYNAMIC_ANCIL_DIR', 
                                                                     check_write=False,
                                                                     default_value=CSPP_RT_ANC_CACHE_DIR)

    default = os.path.join(cspp_home, "anc","static")
    CSPP_RT_ANC_HOME = check_and_convert_env_var(
        'CSPP_RT_ANC_HOME', check_write=False, default_value=default)

    default = "http://jpssdb.ssec.wisc.edu/cspp_v_3_0/ancillary/LUTS_V_3_0/"
    JPSS_REMOTE_ANC_DIR = check_existing_env_var(
        "JPSS_REMOTE_ANC_DIR", default_value=default)


def check_env():
    " Check that needed environment variables are set"

    for key in iter(EXTERNAL_BINARY.keys()):
        if not _ldd_verify(EXTERNAL_BINARY[key]):
            LOG.warning(
                "%r executable is unlikely to run, is LD_LIBRARY_PATH set?" % EXTERNAL_BINARY[key])


logging_configured = False


def configure_logging(level=logging.WARNING, FILE=None):
    "route logging INFO and DEBUG to stdout instead of stderr, affects entire application"
    global logging_configured
    # create a formatter to be used across everything
    # fm = logging.Formatter('%(levelname)s:%(name)s:%(msg)s') #
    # [%(filename)s:%(lineno)d]')
    fm = logging.Formatter(
        '(%(levelname)s):%(filename)s:%(funcName)s:%(lineno)d:%(message)s')

    rootLogger = logging.getLogger()

    # set up the default logging
    if logging_configured is False:
        logging_configured = True

        # create a handler which routes info and debug to stdout with std
        # formatting
        h1 = logging.StreamHandler(sys.stdout)
        f1 = SingleLevelFilter([logging.INFO, logging.DEBUG], False)
        h1.addFilter(f1)
        h1.setFormatter(fm)

        # create a second stream handler which sends everything else to stderr
        # with std formatting
        h2 = logging.StreamHandler(sys.stderr)
        f2 = SingleLevelFilter([logging.INFO, logging.DEBUG], True)
        h2.addFilter(f2)
        h2.setFormatter(fm)
        rootLogger.addHandler(h1)
        rootLogger.addHandler(h2)

    h3 = None
    if FILE is not None:
        work_dir = os.path.dirname(FILE)
        check_and_convert_path("WORKDIR", work_dir, check_write=True)
        h3 = logging.FileHandler(filename=FILE)
        #        f3 = SingleLevelFilter([logging.INFO, logging.DEBUG], False)
        #        h3.addFilter(f3)
        h3.setFormatter(fm)
        rootLogger.addHandler(h3)

    rootLogger.setLevel(level)


def _test_logging():
    LOG.debug('debug message')
    LOG.info('info message')
    LOG.warning('warning message')
    LOG.error('error message')
    LOG.critical('critical message')


def status_line(status):
    """
    Make comment standout in log
    """
    LOG.info("\n                 ( " + status + " )\n")


def env(**kv):
    "augment environment with new values"
    zult = dict(os.environ)
    zult.update(kv)
    return zult


def profiled_sh(cmd, log_execution=True, *args, **kwargs):
    """
    like subprocess.check_call, but returning the pid the process was given and
    logging as INFO the final content of /proc/PID/stat

    """
    pop = Popen(cmd, *args, **kwargs)
    pid = pop.pid
    fn = '/proc/%d/status' % pid
    LOG.debug('retrieving %s statistics to caller dictionary' % fn)
    proc_stats = '-- no /proc/PID/status data --'

    startTime = time.time()
    while True:
        time.sleep(1.0)

        rc = pop.poll()
        if rc is not None:
            break

        try:
            proc = file(fn, 'rt')
            proc_stats = proc.read()
            proc.close()
            del proc
        except IOError as oops:
            LOG.warning('unable to get stats from %s' % fn)

    endTime = time.time()
    delta = endTime - startTime
    LOG.debug('statistics for "%s"' % ' '.join(cmd))

    if log_execution is True:
        status_line('Execution Time:  "%f" Sec Cmd "%s"' %
                    (delta, ' '.join(cmd)))

    LOG.debug(proc_stats)

    if rc != 0:
        exc = CalledProcessError(rc, cmd)
        exc.pid = pid
        raise exc

    return pid


def simple_sh(cmd, log_execution=True, *args, **kwargs):
    LOG.debug('executing %r with kv=%s' % (cmd, pp.pformat(args)))
    
    pop = None
    if STRACE_ENABLED:
        strace = open('strace.log', 'at')
        # print("= " * 32, file=strace)
        # print(repr(cmd), file=strace)
        cmd = ['strace'] + list(cmd)
        pop = Popen(cmd, *args, stderr=strace, **kwargs)
    else:
        pop = Popen(cmd, *args, stderr=PIPE, **kwargs)

    pid = pop.pid
    startTime = time.time()
    
    anc_stderr = pop.communicate()
    rc = pop.returncode

    if rc != 0:
        LOG.error(anc_stderr)
    # rc = pop.wait()

    endTime = time.time()
    delta = endTime - startTime
    LOG.debug('statistics for "%s"' % ' '.join(cmd))
    if log_execution is True:
        now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        status_line('%s Execution Time: %f Sec Cmd "%s"' % (now,delta,cmd))
    if rc != 0:
        exc = CalledProcessError(rc, cmd)
        exc.pid = pid
        raise exc

    return pid


# default sh() is to profile on linux systems
if os.path.exists('/proc') and PROFILING_ENABLED:
    sh = profiled_sh
else:
    sh = simple_sh


def execute_binary_captured_io(work_dir, cmd, log_execution=True, **kv):
    """
    Execute the specifed ancillary script.
    process the ouptut and return the file names.
    """

    LOG.debug('executing %r with kv=%s' % (cmd, pp.pformat(kv)))
    pop = Popen(cmd,
                cwd=work_dir,
                env=env(**kv),
                shell=True,
                stdin=PIPE,
                stdout=PIPE,
                stderr=PIPE,
                close_fds=True)

    startTime = time.time()
    
    anc_stdout, anc_stderr = pop.communicate()

    endTime = time.time()
    delta = endTime - startTime
    LOG.debug('statistics for "%s"' % cmd)
    if log_execution is True:
        now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

        status_line('%s Execution Time: %f Sec Cmd "%s"' %
                    (now,delta, cmd.split(' ')[0]))

    rc = pop.returncode

    if rc == 0:
        LOG.debug("Success " + cmd)

        LOG.debug(anc_stdout.strip().decode('utf8'))
        return anc_stdout.strip().decode('utf8')
    elif rc == 1:
        LOG.info(anc_stdout.decode('utf8'))
        LOG.info(anc_stderr.decode('utf8'))
        LOG.error("stderr:" + anc_stderr.decode('utf8'))
    else:
        LOG.warn("what " + cmd)

    if rc != 0:
        LOG.debug(anc_stdout.decode('utf8'))
        LOG.error(anc_stderr.decode('utf8'))

    return None


def unpack(work_dir, h5,silent=False):
    """run Adl_Unpacker.exe on an HDF5 file with output to a work_dir
    raises EnvironmentError if work_dir is not writable,
           IOError if file is not HDF5 (according to h5py.is_hdf5),
           CalledProcessError if ADL_Unpacker exits with bad return code or segfault
    """
    if not h5.endswith('.h5'):
        LOG.warning('%r should end with .h5?' % h5)
    # check that we have write access to the work directory
    if not os.access(work_dir, os.R_OK | os.W_OK):
        err = 'work directory %r is not writable by this user.' % work_dir
        LOG.error(err)
        raise EnvironmentError(err)
    if not h5py.is_hdf5(h5):
        err = '%r is not a valid HDF5 file (check for incomplete data transfers?)' % h5
        LOG.error(err)
        raise IOError(err)
    LOG.debug('running ADL_Unpacker on %r' % h5)
    LOG.debug("ADL_HOME %s" % (ADL_VARS['ADL_HOME']))
    LOG.debug("DCONFIG %s" % (ADL_VARS['DCONFIG']))
    LOG.debug("WORK_DIR %s" % work_dir)

    cmd = [EXTERNAL_BINARY['ADL_UNPACKER'], h5]
    cmd = " ".join(cmd)
    execute_binary_captured_io(work_dir, cmd, log_execution=not silent, **ADL_VARS)


def ingmsd(work_dir):
    """run Adl_Unpacker.exe on an HDF5 file with output to a work_dir
    raises EnvironmentError if work_dir is not writable,
           IOError if file is not HDF5 (according to h5py.is_hdf5),
           CalledProcessError if ADL_Unpacker exits with bad return code or segfault
    """

    msd_dir = os.path.join(work_dir, 'msd')
    if not os.path.exists(os.path.join(msd_dir, "TILED")):
        os.makedirs(os.path.join(msd_dir, "TILED"))
    if not os.path.exists(os.path.join(msd_dir, "Temp")):
        os.makedirs(os.path.join(msd_dir, "Temp"))
    if not os.path.exists(os.path.join(msd_dir, "ANC")):
        os.makedirs(os.path.join(msd_dir, "ANC"))
    if not os.path.exists(os.path.join(msd_dir, "AUX")):
        os.makedirs(os.path.join(msd_dir, "AUX"))

    # check that we have write access to the work directory
    if not os.access(work_dir, os.R_OK | os.W_OK):
        err = 'work directory %r is not writable by this user.' % work_dir
        LOG.error(err)
        raise EnvironmentError(err)

    LOG.debug("ADL_HOME %s" % (ADL_VARS['ADL_HOME']))
    LOG.debug("DCONFIG %s" % (ADL_VARS['DCONFIG']))
    LOG.debug("WORK_DIR %s" % (work_dir))

    args = '%s/cfg/dynamic/IngMsdLwFile.xml' % ADL_VARS['ADL_HOME']

    LOG.debug('args %s' % args)

    cmd = [EXTERNAL_BINARY['INGMSD'], '-lw', args]
    cmd = " ".join(cmd)
    execute_binary_captured_io(
        work_dir, cmd, WORK_DIR=work_dir, log_execution=True, **ADL_VARS)

    # shutil.rmtree(msd_dir, ignore_errors=True)
def dmhmicli(work_dir,thefiles):
    """run Adl_Unpacker.exe on an HDF5 file with output to a work_dir
    raises EnvironmentError if work_dir is not writable,
           IOError if file is not HDF5 (according to h5py.is_hdf5),
           CalledProcessError if ADL_Unpacker exits with bad return code or segfault
    """

    msd_dir = os.path.join(work_dir, 'msd')
    if not os.path.exists(os.path.join(msd_dir, "TILED")):
        os.makedirs(os.path.join(msd_dir, "TILED"))
    if not os.path.exists(os.path.join(msd_dir, "Temp")):
        os.makedirs(os.path.join(msd_dir, "Temp"))
    if not os.path.exists(os.path.join(msd_dir, "ANC")):
        os.makedirs(os.path.join(msd_dir, "ANC"))
    if not os.path.exists(os.path.join(msd_dir, "AUX")):
        os.makedirs(os.path.join(msd_dir, "AUX"))

    # check that we have write access to the work directory
    if not os.access(work_dir, os.R_OK | os.W_OK):
        err = 'work directory %r is not writable by this user.' % work_dir
        LOG.error(err)
        raise EnvironmentError(err)

    LOG.debug("ADL_HOME %s" % (ADL_VARS['ADL_HOME']))
    LOG.debug("DCONFIG %s" % (ADL_VARS['DCONFIG']))
    LOG.debug("WORK_DIR %s" % (work_dir))

    args = '%s/cfg/dynamic/IngMsdLwFile.xml' % ADL_VARS['ADL_HOME']

    LOG.debug('args %s' % args)

    cmd = [EXTERNAL_BINARY['DBHMICLI'], '%s -full -noFileCopy <<STDIN\n6\n%s\n99\n'%(ADL_VARS['DPE_DMS_INST_PATH'],thefiles) ]
    cmd = " ".join(cmd)
    execute_binary_captured_io(
        work_dir, cmd, WORK_DIR=work_dir, log_execution=True, **ADL_VARS)

    # shutil.rmtree(msd_dir, ignore_errors=True)



def uridcreator(work_dir):
    """run Adl_Unpacker.exe on an HDF5 file with output to a work_dir
    raises EnvironmentError if work_dir is not writable,
           IOError if file is not HDF5 (according to h5py.is_hdf5),
           CalledProcessError if ADL_Unpacker exits with bad return code or segfault
    """

    LOG.debug("ADL_HOME %s" % (ADL_VARS['ADL_HOME']))
    LOG.debug("DCONFIG %s" % (ADL_VARS['DCONFIG']))
    LOG.debug("WORK_DIR %s" % (work_dir))

    args = '%s/cfg/dynamic/IngMsdLwFile.xml' % ADL_VARS['ADL_HOME']

    LOG.debug('args %s' % args)

    cmd = [EXTERNAL_BINARY['URIDCREATOR']]
    cmd = " ".join(cmd)
    urid_str = execute_binary_captured_io(
        work_dir, cmd, WORK_DIR=work_dir, log_execution=False, **ADL_VARS)
    return urid_str.split(' ')[0].split('=')[1]

# def granid(work_dir,gid):
#     """run Adl_Unpacker.exe on an HDF5 file with output to a work_dir
#     raises EnvironmentError if work_dir is not writable,
#            IOError if file is not HDF5 (according to h5py.is_hdf5),
#            CalledProcessError if ADL_Unpacker exits with bad return code or segfault
#     """
# 
#     LOG.debug("ADL_HOME %s" % (ADL_VARS['ADL_HOME']))
#     LOG.debug("DCONFIG %s" % (ADL_VARS['DCONFIG']))
#     LOG.debug("WORK_DIR %s" % (work_dir))
# 
#     #args = '%s/cfg/dynamic/IngMsdLwFile.xml' % ADL_VARS['ADL_HOME']
#     args= ""
# 
#     #cmd = [EXTERNAL_BINARY['GRANID']]
#     #cmd = " ".join(cmd)
#     
#     cmd = 'echo %s | %s'%(gid,EXTERNAL_BINARY['GRANID'])
#     stext = execute_binary_captured_io(
#         work_dir, cmd, WORK_DIR=work_dir, log_execution=True, **ADL_VARS)
#     
#     LOG.debug(stext)
#     
#     tstart = re.search('Start: ([0-9]{16}) = ([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2}).([0-9]{2})', stext)
#     ts='%s%s%s%s'%(tstart.group(5),tstart.group(6),tstart.group(7),tstart.group(8))
#     
#     tend = re.search('End: ([0-9]{16}) = ([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2}).([0-9]{2})', stext)
#     te='%s%s%s%s'%(tend.group(5),tend.group(6),tend.group(7),tend.group(8))
#  
#     istart=int(tstart.group(1))
#     iend = int(tend.group(1))
#     
#     tstart = re.search('Start: [0-9]{16} = ([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}.[0-9]{2})', stext)
#     gran_start_dt = dt.datetime.strptime( tstart.group(1) ,'%Y-%m-%d %H:%M:%S.%f')
#  
#     tend = re.search('End: [0-9]{16} = ([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}.[0-9]{2})', stext)
#     gran_end_dt = dt.datetime.strptime( tend.group(1) ,'%Y-%m-%d %H:%M:%S.%f')
# 
#  
#     return  int(istart), int(iend), iend-istart , gran_start_dt , gran_end_dt



def unpack_h5s(work_dir, h5_names):
    "unpack multiple H5 files into workspace, returning number of problematic files"
    unpacking_problems = 0
    # unpack HDF5 to work directory
    for fn in h5_names:
        try:
            unpack(work_dir, fn)
        except (IOError, EnvironmentError, CalledProcessError) as oops:
            LOG.debug(traceback.format_exc())
            LOG.error('ADL_Unpacker failed on %r: %r . Continuing' %
                      (fn, oops))
            unpacking_problems += 1
    return unpacking_problems


def anc_files_needed(glob_list, search_path_list, optional=False):
    """find all the ancillary files that we need as a sequence to feed to 
    link_ancillary_to_work_dir"""
    for pat in glob_list:
        nfound = 0
        for search_dir in search_path_list:
            if not search_dir:
                continue
            LOG.debug("searching %s for %r" % (search_dir, pat))
            for ancfile in glob.glob(os.path.join(search_dir, pat)):
                nfound += 1
                LOG.debug('found %r' % ancfile)
                yield ancfile
                ascfile = corresponding_asc_path(ancfile)
                if ancfile == "NoneType" or not os.path.isfile(ascfile):
                    LOG.error(
                        'expected %r metadata for %r - missing!' % (ascfile, ancfile))
                    continue
                yield ascfile
        if nfound == 0 and not optional:
            err = 'no ancillary data found for pattern %r' % pat
            LOG.error("Check your anc/static and anc/cache directories")
            LOG.error('Specifed search path is')
            LOG.error(search_path_list)
            LOG.error(err)
            raise EnvironmentError(err)
        else:
            LOG.debug('found %d files for %r' % (nfound, pat))


def link_ancillary_to_work_dir(work_dir, anc_path_seq):
    """link ancillary files into work directory"""
    for src_path in anc_path_seq:
        _, src_name = os.path.split(src_path)
        tgt_path = os.path.abspath(os.path.join(work_dir, src_name))

        if not os.path.exists(tgt_path):
            LOG.debug('%r -> %r' % (src_name, src_path))
            os.symlink(src_path, tgt_path)
        else:
            LOG.info('%r already exists; continuing' % tgt_path)
        try:
            LOG.debug('testing %r' % tgt_path)
            # this checks to make sure exception is not thrown
            os.stat(tgt_path)
        except OSError:
            LOG.error("link at %r is broken" % tgt_path)
            raise


def missing_granules_check(desired_granule_seq, found_granule_seq):
    """verify that granules matching an incoming sequence and specification were created
    returns sequence of granules that are missing
    example: missing_granules_check(work_dir, my_source_granules, 
                adl_asc.skim_dir(work_dir, n_collection_short_name='ATMS-SDR'))
    """
    found = dict((g['N_Granule_ID'], g) for g in found_granule_seq)
    for wanted in desired_granule_seq:
        name = wanted['N_Granule_ID']
        if name not in found:
            LOG.debug('could not find granule for %s' % name)
            yield wanted
        else:
            LOG.debug('found corresponding granule for %s' % name)


def h5_xdr_inventory(glob_pattern, attr_path, state=None):
    """
    returns (new-granules, new-state)
    new-granules is a dictionary of {granule_id:filename, ...} for granule files 
    not seen already as part of state.
    state is an optional input {filename: granule-id, ...} which is also returned
    """
    if state is None:
        state = dict()
    new = dict()
    x = attr_path.split('/')
    for filename in glob.glob(glob_pattern):
        if filename not in state:
            h5 = h5py.File(filename, 'r')
            hob = h5
            for oname in x[:-1]:
                hob = hob[oname]
            granid = hob.attrs[x[-1]][0][0]
            new[granid] = filename
            state[filename] = granid
            LOG.debug('%s found in %s' % (granid, filename))
            h5.close()
            del h5
    return new, state


def get_return_code(num_unpacking_problems, num_xml_files_to_process,
                    num_no_output_runs, noncritical_problem, environment_error):
    """
    based on problems encountered, print final disposition message, return 
    return code to be passed back to caller. Non-zero return code indicates a 
    critical problem was encountered.
    """
    # considered a noncritical problem if there were any runs that crashed,
    # produced no output, where Geo failed, where ADL logs indicated a problem,
    # or where output SDRs failed the imaginary quality check

    # critical problems: set non-zero return code and log error messages
    rc = 0
    if num_unpacking_problems > 0:
        rc |= 2
        LOG.error('Failed to unpack input data.')
    if num_xml_files_to_process and (
            num_xml_files_to_process <= num_no_output_runs):  # skipping this check if no XML files to process
        rc |= 1
        LOG.error('Failed to generate any SDR granules.')
    if environment_error:
        rc |= 8
        LOG.error("Environment error.")

    # if critical error was encountered, print failure message and return
    # error code
    if rc != 0:
        LOG.error('Failure. Refer to previous error messages')
        LOG.info('Failure. Refer to previous error messages')
        return rc

    # otherwise no errors or only non-critical errors: print success message
    # and return 0
    if noncritical_problem:
        LOG.info(
            'Normal Completion. Encountered some problems (refer to previous error messages).')
    else:
        LOG.info('Normal Completion.')
    return rc

def check_static():
    LOG.debug('home %s '%CSPP_RT_HOME)


    db_path = get_shipped_db(CSPP_RT_HOME)
    if not os.path.exists(db_path):
            LOG.error('Missing db is cache installed?')

    if not os.path.exists(os.path.join(CSPP_RT_HOME,'anc','static','swapped')):
            LOG.error('Missing tiles is static package installed?')




configure_logging()
# paths for ADL and ancillary are set to default values based on relative
# location to this module.
cspp_x_home = what_package_am_i()

CSPP_RT_HOME = check_and_convert_env_var(
    "CSPP_SDR_HOME", check_write=False, default_value=cspp_x_home)

ancillary_static_needed = True
gtm_package = cspp_x_home.find("GTM")
if gtm_package != -1:
    LOG.info("No static ancillary required")
    ancillary_static_needed = False
    

initialize_cspp_variables(CSPP_RT_HOME)
initialize_ADL_variables(CSPP_RT_HOME)



if __name__ == '__main__':
    """CSPP_RT_ANC_PATH=/tmp CSPP_RT_ANC_HOME=/tmp CSPP_RT_ANC_CACHE_DIR=/tmp 
       CSPP_RT_ANC_TILE_PATH=/tmp ADL_HOME=/tmp CSPP_RT_HOME=/tmp 
       python adl_common.py >stdout 2>stderr"""
    # logging.basicConfig(level=logging.DEBUG) we don't want basicConfig
    # anymore
    configure_logging(level=logging.DEBUG, FILE="testlog.log")
    _test_logging()
