#!/usr/bin/env python

"""Determine NPP orbit number for a granule"""

from argparse import ArgumentParser
from datetime import datetime
import os
import re
import shutil
import subprocess
import tempfile
import logging
LOG = logging.getLogger(__name__)
class NppOrbNum(object):

    spcrft='NPP'
    wd=None
    def __init__(self,spcrft,wd):
        self.spcrft=spcrft
        self.wd=wd
        self.file_name_parser = NppFileNameParser()

    def main(self):

        args = self.parse_args()
        begin_time = self.file_name_parser.time_range(args.npp_file)[0]
        output = self.execute_where(self.spcrft, args.tle_file,  begin_time)
        orb_num = self.parse_orbit_number_from_where_output(output)
        #print ( orb_num )

    def use(self, npp_file, tle_file):
        begin_time = self.file_name_parser.time_range(npp_file)[0]
        output = self.execute_where(self.spcrft, tle_file,  begin_time)
        orb_num = self.parse_orbit_number_from_where_output(output)
        return orb_num



    def usedt(self, begin_datetime, tle_file):
        output="NO"
        try :

            output = self.execute_where(self.spcrft, tle_file,  begin_datetime)
            LOG.debug(output)
            output8= output.decode("utf-8")
            orb_num = self.parse_orbit_number_from_where_output(output8)
            orbit_number=int(orb_num+1)
        except :
            LOG.info('output %s'%output)
            orbit_number = 1

        return orbit_number

    def parse_args(self):

        parser = ArgumentParser(description=__doc__)
        parser.add_argument('tle_file',
                            help='NORAD two line elements')
        parser.add_argument('npp_file',
                            help='NPP file name; used only to get begin time')
        return parser.parse_args()

    def execute_where(self, satellite, tle_file, time):

        #with TemporaryDirectory(dir=self.wd) as tmp_dir:
            
        tmp_dir=self.wd   
        self.write_tlinset_file(satellite, tle_file, tmp_dir)
        return subprocess.check_output([self.where_binary(), satellite,
                                            time.strftime('%y%m%d'),
                                            time.strftime('%H%M%S')],
                                           cwd=tmp_dir)

    def parse_orbit_number_from_where_output(self, output):
        

        on = re.search('Orbit number:\s+[0-9]+', output)
        try :
            onum=int(on.group(0).replace("Orbit number:",""))

            if onum == -999 :
                onum = 1
        except :
            onum=1

        return  int(onum)
 #       return int(re.search('Orbit number:\s+(\d+)', output).group(1))

    def write_tlinset_file(self, satellite, tle_file_name, tmp_dir):

        with open(os.path.join(tmp_dir, 'TLINSET'), 'w') as tlinset_file:
            tlinset_file.write(satellite + '\n')
            with open(tle_file_name) as tle_file:
                tlinset_file.write(tle_file.read())

    def where_binary(self):

        return os.path.abspath(os.path.join(os.path.dirname(__file__), 'where'))

class NppFileNameParser(object):

    def time_range(self, file_name):

        day, begin_time, end_time = self.extract_time_range_tokens(file_name)
        return [self.time_from_tokens(day, t) for t in (begin_time, end_time)]

    def extract_time_range_tokens(self, file_name):

        return file_name.split('_')[2:5]

    def time_from_tokens(self, day, time):

        return datetime.strptime(day[1:] + time[1:], '%Y%m%d%H%M%S%f')

class TemporaryDirectory(object):

    """Directory that automatically cleans itself up

    This mimics tempfile.TemporaryDirectory from Python 3.2, which is not in
    the Python 2 standard library.
    """

    def __init__(self, dir=None):

        self.name = tempfile.mkdtemp(dir=dir)

    def cleanup(self):

        shutil.rmtree(self.name)

    def __enter__(self):

        return self.name

    def __exit__(self, exc_type, exc_value, traceback):

        self.cleanup()

if __name__ == '__main__':
    NppOrbNum().main()

