#!/usr/bin/env python
# encoding: utf-8
"""
$Id:$

Purpose: Run the ADL 3.1 CRIS SDR

Input:
    One or more HDF5 CRIS RDR input files, aggregated or single-granule.
    You need four granules before and four granules after a given granule to process.
    A work directory, typically, empty, in which to unpack the granules and generate the output.
    If the work directory specified does not exist, it will be created.

Output:
    ADL CRIS SDR blob files, .lutasc metadata files, and HDF5 output granules will be created.

Details:
    If you input a series of granules, the software will scan the work directory.
    Thus, for N input RDR granules, you may get up to N-8 output SDR granules if they are all contiguous.
    It is ambiguous to provide several copies of the same granule in the work directory;
    this will result in an error abort.
    The unpacker gives each unpacking of a given granule its own

    Repairs .lutasc files if necessary to correct the following issues found in output
    from RTPTP 5.0:
       - n_collection_short_name 'CrIS-SCIENCE-RDR' (should be 'CRIS-SCIENCE-RDR')
       - Invalid granule IDs (times are out of range tolerated by ADL)

Preconditions:
    Requires ADL_HOME is set.
    Requires that any needed LD_LIBRARY_PATH is set.

Created 2011 G Martin <gmartin@ssec.wisc.edu>
Copyright (C) 2011-2013 University of Wisconsin Regents.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import sys
import logging
import glob
import traceback
import shutil
import time
import datetime as dt
from subprocess import CalledProcessError, call
from shutil import rmtree
from sdr_common import add_geo_attribute_to_h5, rename_products
# skim and convert routines for reading .lutasc metadata fields of interest

from adl_common import check_anc_cache, ADL_HOME, CSPP_RT_ANC_CACHE_DIR, COMMON_LOG_CHECK_TABLE, sh
from adl_common import env, check_static
from adl_common import link_ancillary_to_work_dir, anc_files_needed, configure_logging, h5_xdr_inventory, get_return_code, check_env
from adl_common import check_existing_env_var
from adl_common import check_and_convert_path, status_line, link_ancillary_to_work_dir
from adl_common import CSPP_RT_HOME, initialize_work_ADL_variables, ADL_VARS
from sdr_common import cleanup_work_dir
# cache loading and searching
import rdr_unpack
import datetime
# N_GEO_Ref fix for ADL
# from adl_geo_ref import write_geo_ref

import adl_log

from qc_cris_sdr import imaginary_threshold_check
from adl_post_process import repack_products, aggregate_products_par, SHORTNAME_2_PRODUCTID, aggregate_products
from multiprocessing import Pool

from adl_cris_data import GIG_MEMORY_PER_PROC, CRIS_GRAN_ID_DELTA
from sdr_common import check_products,  build_workspace, setup_multi_process, trim_the_granules, \
    start_out_good, report_gran_completion, \
    number_good_granules, check_memory, check_number_processors

from sdr_common import move_products_to_work_directory
from adl_query_dms import skim_dir, setup_dms_in_work_directory
import adl_anc_retrieval
import pprint
from sdr_common import abs_filenames
pp = pprint.PrettyPrinter(indent=4)

# every module should have a LOG object
LOG = logging.getLogger(__name__)

# keys used in metadata dictionaries
K_FILENAME = '_asc_filename'

ADL_CRIS_SDR = os.path.join(ADL_HOME, 'bin', 'ProSdrCrisController.exe')
ADL_CRIS_FS_SDR = os.path.join(ADL_HOME, 'bin', 'ProSdrCrisFSController.exe')

# products which we aggregate
CRIS_SDR_PRODUCTS_AGG = [
    'CrIS-SDR'
]

#CrIS-FS-Correct-Matrix
extra_prod = [
    'CrIS-Correct-Matrix',
    'CrIS-SDR-ENGPKT-BACKUP',
    'CrIS-FS-Correct-Matrix',
    'CrIS-FS-SDR-ENGPKT-BACKUP'
]
# products which we aggregate and assign N_GEO_Ref to
CRIS_SDR_PRODUCTS_AGG_GEO = [
    'CrIS-SDR-GEO'
]

# products which we aggregate
FS_CRIS_SDR_PRODUCTS_AGG = [
    'CrIS-FS-SDR'

]

# products which we aggregate and assign N_GEO_Ref to
FS_CRIS_SDR_PRODUCTS_AGG_GEO = [
    'CrIS-SDR-GEO'
]


# optional CRIS CMO input glob
# Note: this glob is a hack: want to exclude .h5 and .H5 only but this excludes
# .h? and .H?. Should be fine for our purposes
# FUTURE: support H5 CMO ingest?
ADL_CRIS_CMO_GLOBS = ('*CrIS-Correct-Matrix-AUX*[!hH]?',)
# filenames which we'll need to set N_GEO_Ref on
ADL_CRIS_SDR_GLOBS = ('SCRIS*.h5',)

# attribute paths for SDRs and TDRs
SDR_GRANULE_ID_ATTR_PATH = 'Data_Products/CrIS-SDR/CrIS-SDR_Gran_0/N_Granule_ID'

# table of Cris SDR -specific ADL log error messages and the associated
# hint to correcting the problem
CRIS_SDR_LOG_CHECK_TABLE = [
    ("Unable to find start of scan from file ProSdrCris.cpp",
     "Problem with RDR data, possible no Earth scenes"),
]

MINIMUM_CRIS_RDR_BLOB_SIZE = 6000000  # in bytes

# tolerance experimentally found to allow output from RTSTPS 5.3 with bad
# granule times to be processed
MAX_CONTIGUOUS_DELTA_CRIS_RTSTPS_5_3 = 30  # in seconds


def _ldd_verify(exe):
    "check that a program is ready to run"
    rc = call(['ldd', exe], stdout=os.tmpfile(), stderr=os.tmpfile())
    return (rc == 0)


# XML template for ProSdrCrisController.exe
XML_TMPL_CRIS_SDR_full = """
<?xml version="1.0" encoding="UTF-8"?>
<!--
  Config file to run a WFM-managed process in lightweight (no WFM) mode.

  Run a WFM-managed process with no args for help on the format of this file.
-->
<InfWfm_TkLwConfig xmlns="http://jpss.lwtkcfg.ray.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://jpss.lwtkcfg.ray.com ExecutionConfig.xsd">
  <processName>ProSdrCrisController.exe</processName>
  <perfEnable>FALSE</perfEnable>
  <perfPath>${GRANULE_OUTPUT_DIR}</perfPath>
  <debugDest>D_FILE</debugDest>
  <debugLevel>DBG_HIGH</debugLevel>
  <debugPath>${GRANULE_OUTPUT_DIR}</debugPath>
  <rootDir>${GRANULE_OUTPUT_DIR}</rootDir>
  <startMode>INF_STARTMODE_COLD</startMode>
  <healthInterval>30</healthInterval>
  <configGuideName>ProSdrCrisController_GuideList.cfg</configGuideName>

  <task>
    <taskType>GRANULE</taskType>
    <granuleId>%(N_Granule_ID)s</granuleId>
    <granuleVersion>A1</granuleVersion>
    <spacecraft>${SPCRFT}</spacecraft>
    <sensor>CRIS</sensor>
  </task>
  

  

  <task>
    <taskType>SHUTDOWN</taskType>
  </task>
</InfWfm_TkLwConfig>

"""
XML_TMPL_CRIS_FS_SDR_full = """
<?xml version="1.0" encoding="UTF-8"?>
<!--
  Config file to run a WFM-managed process in lightweight (no WFM) mode.

  Run a WFM-managed process with no args for help on the format of this file.
-->
<InfWfm_TkLwConfig xmlns="http://jpss.lwtkcfg.ray.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://jpss.lwtkcfg.ray.com ExecutionConfig.xsd">
  <processName>ProSdrCrisFSController.exe</processName>
  <perfEnable>FALSE</perfEnable>
  <perfPath>${GRANULE_OUTPUT_DIR}</perfPath>
  <debugDest>D_FILE</debugDest>
  <debugLevel>DBG_HIGH</debugLevel>
  <debugPath>${GRANULE_OUTPUT_DIR}</debugPath>
  <rootDir>${GRANULE_OUTPUT_DIR}</rootDir>
  <startMode>INF_STARTMODE_COLD</startMode>
  <healthInterval>30</healthInterval>
  <configGuideName>ProSdrCrisFSController_GuideList.cfg</configGuideName>

  <task>
    <taskType>GRANULE</taskType>
    <granuleId>%(N_Granule_ID)s</granuleId>
    <granuleVersion>A1</granuleVersion>
    <spacecraft>${SPCRFT}</spacecraft>
    <sensor>CRIS</sensor>
  </task>
  
  
    <task>
    <taskType>GRANULE</taskType>
    <granuleId>%(N_Granule_ID)s</granuleId>
    <granuleVersion>A1</granuleVersion>
    <spacecraft>${SPCRFT}</spacecraft>
    <sensor>CRIS</sensor>
  </task>

  <task>
    <taskType>SHUTDOWN</taskType>
  </task>
</InfWfm_TkLwConfig>

"""

XML_TMPL_CRIS_FS_SDR = """
<?xml version="1.0" encoding="UTF-8"?>
<!--
  Config file to run a WFM-managed process in lightweight (no WFM) mode.

  Run a WFM-managed process with no args for help on the format of this file.
-->
<InfWfm_TkLwConfig xmlns="http://jpss.lwtkcfg.ray.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://jpss.lwtkcfg.ray.com ExecutionConfig.xsd">
  <processName>ProSdrCrisFSController.exe</processName>
  <perfEnable>FALSE</perfEnable>
  <perfPath>${GRANULE_OUTPUT_DIR}</perfPath>
  <debugDest>D_FILE</debugDest>
  <debugLevel>DBG_HIGH</debugLevel>
  <debugPath>${GRANULE_OUTPUT_DIR}</debugPath>
  <rootDir>${GRANULE_OUTPUT_DIR}</rootDir>
  <startMode>INF_STARTMODE_COLD</startMode>
  <healthInterval>30</healthInterval>
  <configGuideName>ProSdrCrisFSController_GuideList.cfg</configGuideName>
"""
XML_TMPL_CRIS_SDR = """
<?xml version="1.0" encoding="UTF-8"?>
<!--
  Config file to run a WFM-managed process in lightweight (no WFM) mode.

  Run a WFM-managed process with no args for help on the format of this file.
-->
<InfWfm_TkLwConfig xmlns="http://jpss.lwtkcfg.ray.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://jpss.lwtkcfg.ray.com ExecutionConfig.xsd">
  <processName>ProSdrCrisController.exe</processName>
  <perfEnable>FALSE</perfEnable>
  <perfPath>${GRANULE_OUTPUT_DIR}</perfPath>
  <debugDest>D_FILE</debugDest>
  <debugLevel>DBG_HIGH</debugLevel>
  <debugPath>${GRANULE_OUTPUT_DIR}</debugPath>
  <rootDir>${GRANULE_OUTPUT_DIR}</rootDir>
  <startMode>INF_STARTMODE_COLD</startMode>
  <healthInterval>30</healthInterval>
  <configGuideName>ProSdrCrisController_GuideList.cfg</configGuideName>

"""

def sift_metadata_for_cris_sdr(spcrft, work_dir='.', apply_size_threshold=True):
    """
    search through the ASC metadata in a directory,
    grouping in StartTime order
    look for back-to-back granules and break into contiguous sequences
    yield sequence of granules we can process
    """

    Science_RDRs = list(
        skim_dir(n_collection_short_name="CRIS-SCIENCE-RDR", spcrft=spcrft))

    status_line("Total Science RDRs: " + str(len(Science_RDRs)))
    start_time_key = lambda x: (
        x['StartTime'], x.get('N_Granule_Version', None))

    for group in sorted(Science_RDRs, key=start_time_key):
        yield group

    return

# Look through new log files for completed messages
def checkADLLogForSuccess(work_dir, pid, xml):
    """
    Find the log file
    Look for success
    Return True if found
    Display log message and hint if problem occurred
    """

    # retrieve exe name and log path from lw file
    #logDir = os.path.join(work_dir, "log")
    logExpression = "*" + str(pid) + "*.log"

    files = glob.glob(os.path.join(work_dir, logExpression))
    LOG.info(files)
    zult = True
    for logFile in files:
        LOG.debug("Checking Log file " + logFile + " for errors.")

        n_err = adl_log.scan_log_file(
            COMMON_LOG_CHECK_TABLE + CRIS_SDR_LOG_CHECK_TABLE, logFile)

        if n_err == 0:
            LOG.debug(
                " Log: " + logFile + " " + xml + " Completed successfully")
        else:
            LOG.error(
                " Log: " + logFile + " " + xml + " Completed unsuccessfully")
            zult = False

    return zult


def run_and_check_single_granule(additional_env):
    "run a VIIRS SDR XML input in sequence"

    good = True

    work_dir = additional_env['WORK_DIR']
    xml = additional_env['XML_FILE']
    granule_id = additional_env['N_Granule_ID']
    granule_output_dir = additional_env['GRANULE_OUTPUT_DIR']
    chunk_size=additional_env['CHUNK_SIZE']
    chunks = 0

    satpat = os.path.join(granule_output_dir, 'SCRIS*.h5')
    prior_granules, scris = h5_xdr_inventory(satpat, SDR_GRANULE_ID_ATTR_PATH)
    prior_granules = set(prior_granules.keys())

    # if 'FULLRES' in additional_env :
    # fullres=bool(additional_env['FULLRES'])
    fstr = str(additional_env['FULLRES'])
    fullres = 'True' in fstr

    LOG.debug('%s %s' % (fstr, str(fullres)))
    start_time = time.time()

    if fullres is True:
        cmd = [ADL_CRIS_FS_SDR, xml]
    else:
        cmd = [ADL_CRIS_SDR, xml]

    LOG.debug('executing "%s"' % ' '.join(cmd))
    LOG.debug('additional environment variables: %s' %
              pp.pformat(additional_env))
    try:

        z = ADL_VARS.copy()
        z.update(additional_env)  # which returns None since it mutates z
        LOG.debug(pp.pformat(z))

        pid = sh(cmd, env=env(**z), cwd=work_dir)
        LOG.debug("%r ran as pid %d" % (cmd, pid))

    except CalledProcessError as oops:
        pid = getattr(oops, 'pid', None)

        LOG.debug(traceback.format_exc())
        LOG.error(
            'ProSdrCrisController.exe failed on %r: %r. Continuing...' % (xml, oops))
        good = False
    if fullres is True:
        product_shortnames = FS_CRIS_SDR_PRODUCTS_AGG + \
            FS_CRIS_SDR_PRODUCTS_AGG_GEO

    else:
        product_shortnames = CRIS_SDR_PRODUCTS_AGG + CRIS_SDR_PRODUCTS_AGG_GEO

    problem = check_products(
        granule_output_dir, granule_id, product_shortnames,chunk_size=chunk_size)

    if problem is True:
        good = False

    # check log file created for this PID
    if not checkADLLogForSuccess(granule_output_dir, pid, xml):
        good = False

    product_ids = []

    to_move = product_shortnames
    for short_name in to_move:
        sdr_name = SHORTNAME_2_PRODUCTID[short_name]
        product_ids.append(sdr_name)

    for name in extra_prod:
        product_ids.append(name)

    rename_products(granule_output_dir, product_ids)

    gran_id = additional_env['N_Granule_ID']

    add_geo_attribute_to_h5(
        granule_output_dir, gran_id, CRIS_SDR_PRODUCTS_AGG + FS_CRIS_SDR_PRODUCTS_AGG)

    chunks , option_chunks = move_products_to_work_directory(granule_output_dir, product_ids,optional=extra_prod)

    end_time = time.time()
    elapsed_time = end_time - start_time
    status_line('Tot Execution Time:  "%f Exec/Chunk %f " Gran "%s"' %
                (elapsed_time,elapsed_time/int(chunk_size), gran_id))

    stat = ""
    if good is True:
        stat = 'Granule %s Completed without problems. ' % granule_id

    if good is False:
        stat = 'Granule %s Completed with problems. ' % granule_id
    report_gran_completion(stat, chunks,chunk_size)

    result = {"GOOD": str(good)}

    return result


def run_all_granules(work_dir, granules_to_process, trim=None, setup_only=False, CLEANUP="True", nprocs="1",
                     spcrft='NPP', fullresolution=False,accuracyMode='reduced',extra=False,
                     **additional_env):
    """run each CRIS SDR XML input in sequence
    return info regarding problems encountered: number of bad logs,
    """
    if not trim:
        trim = [4, 0]

    global total_granules
    noncritical_problem = 0

    # obtain pre-existing granule list
    satpat = os.path.join(work_dir, 'SCRIS*.h5')
    prior_granules, scris = h5_xdr_inventory(satpat, SDR_GRANULE_ID_ATTR_PATH)
    prior_granules = set(prior_granules.keys())

    total_granules = len(granules_to_process)

    # build XML configuration files for jobs that can be run

    LOG.info('%d granules to process' % (len(granules_to_process)))

    if len(granules_to_process) == 0:
        return 1

    
    gran = granules_to_process[0]
    gran_id = gran['N_Granule_ID']
    granule_output_dir = os.path.join(work_dir, gran_id)

    if not os.path.isdir(granule_output_dir):
        run_cmo = True
        
    run_cmo =  not extra
 
    # already present in work directory - user supplied
    if fullresolution is True:
        fnxml = 'sdr_cris_%s.xml'
        tmplate = XML_TMPL_CRIS_FS_SDR
        cmo_path = os.path.join(work_dir, "*CrIS-FS-Correct-Matrix*.h5")

    else:
        fnxml = 'sdr_cris_%s.xml'
        tmplate = XML_TMPL_CRIS_SDR
        cmo_path = os.path.join(work_dir, "*CrIS-Correct-Matrix*.h5")

    files = glob.glob(cmo_path)
    if len(files) >= 1:
        run_cmo = False

    granules_to_process = trim_the_granules(
        granules_to_process, CRIS_GRAN_ID_DELTA, trim=trim)

    inproc= int( nprocs )
    chunk_size= int(  (len(granules_to_process) + inproc )  / inproc )
    if chunk_size <= 0:
        chunk_size=1

    additional_keys = dict(SPCRFT=spcrft, FULLRES=str(fullresolution), CHUNK_SIZE=str(chunk_size),INSTRUMENT='CRIS')



    argument_dictionaries, granules_to_process = setup_multi_process(work_dir,
                                                                     granules_to_process,
                                                                     [fnxml],
                                                                     [tmplate],
                                                                     additional_keys,
                                                                     accuracyMode=accuracyMode)

    total_granules = len(granules_to_process)
    start_out_good(total_granules)

    # the first granule needs to be processed before multi-processing so
    # that the CMO gets generated only once.
    # in pipelined operation the CMO may already exits or be in the process
    # of getting created
    # if the lutasc file for the CMO exists the CMO creation is complete, if the directory for the
    # first granule exists does the MO does not we must wait.

    first_granule_cleanup = CLEANUP == str(True)
  
    while run_cmo is True:
        # generate CMO
        if len(argument_dictionaries) > 0:
            cmo_env = argument_dictionaries.pop(0)
            LOG.info(
                "Check CMO with Granule %s Generate if nessesary.", cmo_env['N_Granule_ID'])
            result = run_and_check_single_granule(cmo_env)

            cmo_files = glob.glob(cmo_path)
            if len(cmo_files) >= 1:
                LOG.info('CMO generated %s' % cmo_files[0])
                run_cmo = False

            if result['GOOD'] == "True":
                run_cmo = False
                #2018_01_22_022
                if len(cmo_files) >= 1:
                    cmo_file=os.path.basename(cmo_files[0])
                    cmo_time=cmo_file.split('_')[3][0:8]
                    
                    desired_dt = datetime.datetime.strptime(cmo_time , "%Y%m%d")
                    anc_string = desired_dt.strftime('%Y_%m_%d_%j')
                    cmo_file="CrIS*.h5"
                    status_line('mkdir -p ${CSPP_SDR_HOME}/anc/cache/%s;cp %s ${CSPP_SDR_HOME}/anc/cache/%s '%(anc_string,cmo_file,anc_string) )
                if CLEANUP == str(True):
                    granule_dir = cmo_env['GRANULE_OUTPUT_DIR']
                    LOG.debug("Remove: " + granule_dir)
                    LOG.debug("Remove: " + granule_dir)
                    if os.path.exists(granule_dir):
                        rmtree(granule_dir)
            else:
                LOG.error('Problem with CMO generation granule. %s' %
                          cmo_env['N_Granule_ID'])
                if first_granule_cleanup is True:
                    granule_dir = cmo_env['GRANULE_OUTPUT_DIR']
                    LOG.debug("Remove: " + granule_dir)
                    LOG.debug("Remove: " + granule_dir)
                    if os.path.exists(granule_dir):
                        rmtree(granule_dir)
                    first_granule_cleanup = False
 
                noncritical_problem += 1

        else:
            LOG.error("No CMO generated")
            run_cmo = False

    cmo_path = os.path.join(work_dir, "*CrIS-Correct-Matrix-AUX*.h5")
    cmo_files = glob.glob(cmo_path)

    keep_all_output = False
    results = []
    # results = map (run_and_check_single_viirs_sdr, argument_dictionaries )
    # if there are still granules to process and a cmo has been generated.
    # if len(argument_dictionaries) > 0 and len(cmo_files) > 0:
    if len(argument_dictionaries) > 0:

        nprocs = check_number_processors(int(nprocs))
        gig_recomended = GIG_MEMORY_PER_PROC
        check_memory(nprocs, gig_recomended)

        parallel = Pool(int(nprocs))

        try:
            results = parallel.map_async(
                run_and_check_single_granule, argument_dictionaries).get(9999999)
        except KeyboardInterrupt:
            LOG.error("Got exception, stopping workers and exiting.")
            parallel.terminate()
            parallel.join()
            sys.exit(1)

    for result in results[1:-1]:
        # do not fail on first or last granule - these can have errors
        if result["GOOD"] != "True":
            LOG.debug("result check %s" % result["GOOD"])
            keep_all_output = True
            noncritical_problem += 1

    if CLEANUP == str(True) and keep_all_output is False:
        for args in argument_dictionaries:
            granule_dir = args['GRANULE_OUTPUT_DIR']
            LOG.debug("Remove: " + granule_dir)
            if os.path.exists(granule_dir):
                rmtree(granule_dir)

                # clean up the science data only if you there are no other
                # granule directories left

        remaining_files_and_directories = glob.glob(
            os.path.join(work_dir, "NPP*"))

        if len(remaining_files_and_directories) == 0:
            log_dir = os.path.join(work_dir, "log")
            cris_cleanup(
                work_dir, 'sdr_cris_*.xml', 'ProSdrCrisController.exe_*', log_dir)

    return noncritical_problem


def cris_cleanup(work_dir, xml_glob, log_dir_glob, *more_dirs):
    "upon successful run, clean out work directory"
    # LOG.info("cleaning up work directory...")
    for fn in glob.glob(os.path.join(work_dir, '????????-?????-????????-????????.*')):
        LOG.debug('removing %s' % fn)
        os.unlink(fn)
    for fn in glob.glob(os.path.join(work_dir, xml_glob)):
        LOG.debug('removing task file %s' % fn)
        os.unlink(fn)
    for dirname in glob.glob(os.path.join(work_dir, log_dir_glob)):
        LOG.debug('removing logs in %s' % dirname)
        rmtree(dirname, ignore_errors=True)
    for dirname in more_dirs:
        LOG.debug('removing %s' % dirname)
        rmtree(dirname, ignore_errors=True)


def cris_sdr(work_dir, cris_cmo_dir, h5_names, trim=[3, 3], setup_only=False, allow_cache_update=True, debug=False,
             apply_size_threshold=True, nprocs="1", spcrft='NPP', 
             fullresolution=False, aggregate=False, compress=False,accuracyMode='reduced',center=False,extra=False):
    "process CRIS data to SDRs"

    total_granules = 0
    noncritical_problem = 0
    LOG.debug('trim %s' % (pp.pformat(trim)))
    build_workspace(work_dir)

    num_unpacking_problems = rdr_unpack.unpack_rdr(
        work_dir, h5_names, allow_cache_update=allow_cache_update)

    # read through ascii metadata and build up information table

    granules_to_process = list(
        sift_metadata_for_cris_sdr(spcrft, work_dir, apply_size_threshold=True))
    
    if len(granules_to_process) < 9:
        LOG.warn('Cris needs at least 9 granules only %d available'%len(granules_to_process))
        return 1
    
        # test mode to pick center granules for shortened test cycle will result in one  granule processed
    if center is True: 
        c = int(len(granules_to_process)/2)
        r = int(len(granules_to_process) % 2)
        trim[0] = c
        trim[1] =   ( c - 1  + r )
        #LOG.info(' r %d c %d o %d '%(r,c,(len(granules_to_process) - ( c - r ))))
        LOG.debug(trim)


    LOG.debug(pp.pformat(granules_to_process))
    LOG.debug(', '.join(x['N_Granule_ID'] for x in granules_to_process))
    if not granules_to_process:
        LOG.warning(
            "Found no granules to process. (Need a sequence of at least 9 contiguous granules to "
            "satisfy CrIS SDR cross-granule requirements.)")
        return get_return_code(num_unpacking_problems, num_xml_files_to_process=0, num_no_output_runs=0,
                               noncritical_problem=False, environment_error=False)

    adl_anc_retrieval.service_remote_ancillary(
        work_dir, granules_to_process,
        adl_anc_retrieval.kPOLAR,spcrft,
        allow_cache_update=allow_cache_update)

    adl_anc_retrieval.service_remote_ancillary(
        work_dir, granules_to_process,
        adl_anc_retrieval.kTLE,spcrft,
        allow_cache_update=allow_cache_update)
    
    adl_anc_retrieval.service_remote_ancillary(
        work_dir, granules_to_process,
        adl_anc_retrieval.kCMO,spcrft,
        allow_cache_update=allow_cache_update)
    
    # now update orbit number?
    adl_anc_retrieval.update_orbit_in_db(work_dir, granules_to_process, spcrft)

    # link ancil data
    environment_error = False
    try:

        # run the processing, Note that this will log any non-critical problems
        cleanmeup = str(not debug)
        noncritical_problem = run_all_granules(work_dir, granules_to_process, trim=trim, setup_only=setup_only,
                                               CLEANUP=cleanmeup, WORK_DIR=work_dir,
                                               # LINKED_ANCILLARY=anc_dir,
                                               ADL_HOME=ADL_HOME,
                                               # CSPP_RT_ANC_TILE_PATH=CSPP_RT_ANC_TILE_PATH,
                                               nprocs=nprocs, spcrft=spcrft, fullresolution=fullresolution,
                                               accuracyMode=accuracyMode,extra=extra)


        if fullresolution is True:
            geo = FS_CRIS_SDR_PRODUCTS_AGG_GEO
            addgeo = FS_CRIS_SDR_PRODUCTS_AGG
            # addgeo = CRIS_SDR_PRODUCTS_AGG

        else:
            geo = CRIS_SDR_PRODUCTS_AGG_GEO
            addgeo = CRIS_SDR_PRODUCTS_AGG

        # FIXME: move this to cris_sdr and should add return codes to
        # aggregate_products() and repack_products()
        if aggregate is True:
            # geo first because others reference it
            aggregate_products_par(
                work_dir, geo , nprocs=str(nprocs),spcrft=spcrft)
            aggregate_products_par(
                work_dir, addgeo , nprocs=str(nprocs),spcrft=spcrft)
            add_geo_attribute_to_h5(work_dir, "aggregates", addgeo)
        if compress is True:
            repack_products(work_dir, geo + addgeo)
            
            
        product_ids = []

        to_move = geo + addgeo
        for short_name in to_move:
            sdr_name = SHORTNAME_2_PRODUCTID[short_name]
            product_ids.append(sdr_name)

        for name in extra_prod:
            product_ids.append(name)

        rename_products(work_dir, product_ids)

    except EnvironmentError:
        """
        """
        environment_error = True
        LOG.error(traceback.format_exc())

        # print final disposition message and get return code
        #      rc = get_return_code(num_unpacking_problems, len(granules_to_process), num_no_output_runs=0,
        #  noncritical_problem=False, environment_error=True)

    gran_failure = total_granules - number_good_granules()

    rc = get_return_code(num_unpacking_problems, total_granules,
                         gran_failure, noncritical_problem, environment_error)

    if cleanmeup is True:
        cleanup_work_dir(work_dir,spcrft=spcrft)

    return rc


def main():
    import argparse

    desc = """
Run the CrIS SDR algorithm on the specified HDF5 RDR file(s). Output SDR and
Geo files will go in the specified working directory (defaults to current
directory)
"""
    parser = argparse.ArgumentParser(description=desc)
    #    parser.add_argument('-t', '--test',
    #                action="store_true", default=False, help="run self-tests")
    parser.add_argument('-l', '--local',
                        action="store_true", default=False, help="disable download of remote ancillary data to cache")
    parser.add_argument('-W', '--work-dir', metavar='work_dir', default='.',
                        help='directory in which all activity occurs, defaults to current dir')
    parser.add_argument('-v', '--verbosity', action="count", default=0,
                        help='each occurrence increases verbosity 1 level through ERROR-WARNING-INFO-DEBUG')
    parser.add_argument('-c', '--cris-cmo-dir', metavar='cris_cmo_dir', default='',
                        help='optional, directory containing .bin CrIS CMO file(s)')
    parser.add_argument('-d', '--debug',
                        action="store_true", default=False, help="always retain intermediate files")
    parser.add_argument('-t', '--trim', nargs=2, type=int, metavar='N',
                        default=None,
                        help='Number Trim granules. -t \"Trim Begin\" \"Trim End\" -t 1 1 Trims first and last granule.')

    parser.add_argument('-s', '--no-size-threshold',
                        action="store_true", default=False,
                        help="do not apply size threshold to RDRs. Use at your own risk; may cause ADL to crash or hang.")

#     parser.add_argument('-m', '--accuracymode',
#                         action="store_true", default=False, help="increase geo accuracy")


    parser.add_argument('-z', '--zip',
                        action="store_true", default=False, help="compress products with h5repack zip compression")

    parser.add_argument('-f', '--fsr',
                        action="store_true", default=None, help="FSR res product")

    parser.add_argument('-n', '--nsr',
                        action="store_true", default=None, help="NSR product")

    parser.add_argument('-a', '--aggregate',
                        action="store_true", default=False, help="aggregate products with nagg")
    

    
    parser.add_argument('--center', dest='center', action='store_true',
                        help='Trim to center granule')
    parser.add_argument('--no-center', dest='center', action='store_false',
                        help='Normal output')
    parser.set_defaults(center=False)

    parser.add_argument('-p', '--processors', metavar='n',
                        type=int, default=1, help="Number of processors to use for granule processing, defaults to 1")
    parser.add_argument('filenames', metavar='filename', type=str, nargs='+',
                        help='HDF5 CrIS RDR file to process')

    args = parser.parse_args()

    levels = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]
    level = levels[args.verbosity if args.verbosity < 4 else 3]

    work_dir = check_and_convert_path("WORK_DIR", args.work_dir)
    d = dt.datetime.now()
    timestamp = d.isoformat()
    logname = "cris_sdr." + timestamp + ".log"
    logfile = os.path.join(work_dir, logname)
    configure_logging(level, FILE=logfile)
    LOG.debug('verbosity level is %d' % args.verbosity)

    compress = args.zip
    aggregate = args.aggregate
#     if args.aggregate is True:
#         if len(args.filenames) > 1:
#             LOG.error("Only one input allowed with aggregate option")
#             sys.exit(2)


    accuracyMode='reduced'
#     if args.accuracymode is True:
#         accuracyMode = 'full'




    work_dir = os.path.abspath(args.work_dir)
    cris_cmo_dir = args.cris_cmo_dir
    if cris_cmo_dir:
        cris_cmo_dir = os.path.abspath(cris_cmo_dir)
    if not args.trim:
        args.trim = [3, 3]

    if not args:
        parser.print_help()
        return 9

    dpe_version = check_existing_env_var('DPE_VER', default_value=None)
    LOG.info("product version: DPE_VER for CRIS SDR  %s" % dpe_version)

    check_anc_cache(check_write=not args.local)
    check_static()
    full_db_name = setup_dms_in_work_directory(work_dir, CSPP_RT_HOME)
    initialize_work_ADL_variables(work_dir, full_db_name)

    check_env()

    LOG.info('CSPP execution work directory is %r' % work_dir)

    rc = 1
    try:

        if 'npp' in os.path.basename(args.filenames[0] ):
            spcrft = 'NPP'
            fullres = False
            if args.fsr is not None:
                fullres = True
        else:
            spcrft = 'J01'
            fullres = True
            if args.nsr is not None:
                fullres = False


        if args.nsr is not None and args.fsr is not None :
            LOG.error('Pick FSR or NSR not both')
            sys.exit( 1 )
#       if fullres is True and aggregate is True:
#            LOG.warn('aggregation not supported with full resolution')
#           aggregate = False

        filenames = abs_filenames(args.filenames)

        rc = cris_sdr(work_dir, cris_cmo_dir, filenames, allow_cache_update=not args.local, debug=args.debug,
                      apply_size_threshold=not args.no_size_threshold, trim=args.trim, nprocs=str(args.processors),
                      spcrft=spcrft, fullresolution=fullres, aggregate=aggregate, compress=compress,accuracyMode=accuracyMode,center=args.center,extra=True)

    except Exception:
        LOG.error(traceback.format_exc())

    if not args.debug:
        os.remove(logfile)
        cleanup_work_dir(work_dir,spcrft=spcrft,force_all=True)

    return rc


if __name__ == '__main__':
    sys.exit(main())
